################################################################################
############ Network Inference Using the Hub Model and Variants
############ R code: simulation and data analysis
############ Table 1, Table 2, Table 3, Passerines and Bakery Data  
############ IMPORTANT!!! Please install the R package "Rsolnp"
################################################################################
rm(list=ls())
################################################################################
############ Table 1 & Table 2
library(foreach) ## using parallel computing
#### Table 1 
source("table1_nonull.R") 
out = tbl1.simu(Iter = 5) ## 2-3 minutes
table1 = rbind(cbind(out[1:4,], out[5:8,]), cbind(out[9:12,],out[13:16,]))
## save to a txt file
cat("########################## Network Inference Using Hub Model and Variants ##########################",
    "\n", file = "OneTable.txt", append = F)
cat("\n", file = "OneTable.txt", append = T)
cat("########################## Simulation Study ##########################", 
    "\n", file = "OneTable.txt", append = T)
cat("\n", file='OneTable.txt', append=T)
cat("########################## Table 1 ##########################", 
    "\n", file = "OneTable.txt", append = T)
cat("\n", file='OneTable.txt', append = T)
write.table(round(table1,4), file='OneTable.txt', append=T, sep='\t', row.names=F, col.names=T)
#### Table 2
source("phub.R")
source("table2_null.R")
out = tbl2.simu(Iter = 5) ## 2-3 minutes
table2 = rbind(cbind(out[1:4,], out[5:8,]), cbind(out[9:12,],out[13:16,]))
cat("\n", file='OneTable.txt', append=T)
cat("########################## Table 2 ##########################", "\n", file = "OneTable.txt",append=T)
cat("\n", file='OneTable.txt', append = T)
write.table(round(table2,4), file='OneTable.txt', append=T, sep='\t', row.names=F, col.names=T)
################################################################################
############ Real Data
#### Passerines
source("datahub.R")
passerines = read.csv("passerine.csv", header = F)
table.data1 = data.hub(passerines, M = 55,Lam = seq(0.045,0.065,0.005)) ## full data
table.data1.boot = data.hub(passerines, M = 55,Lam = seq(0.045,0.065,0.005), bootstrap = 10) ## bootstrap
cat("\n", file='OneTable.txt', append = T)
cat("########################## Passerine Data ##########################", "\n", file = "OneTable.txt",append=T)
cat("\n", file='OneTable.txt', append=T)
write.table(table.data1, file='OneTable.txt', append=T, sep='\t', row.names=T, col.names=T)
cat("\n", file='OneTable.txt', append = T)
cat("########################## Passerine Data: Bootstrap ##########################","\n", file = "OneTable.txt",append=T)
cat("\n", file='OneTable.txt', append=T)
write.table(table.data1.boot, file='OneTable.txt', append=T, sep='\t', row.names=F, col.names=T)
#### Bakery Store
bakery5000 = read.csv("bakery_5000.csv", header = F, row.names = 1)
table.data2 = data.hub(data = bakery5000, M = 40, Lam = seq(0.015,0.035,0.005)) ## full data
cat("\n", file='OneTable.txt', append = T)
cat("########################## Bakery Data ##########################", "\n", file = "OneTable.txt",append=T)
cat("\n", file='OneTable.txt', append=T)
write.table(table.data2, file='OneTable.txt', append=T, sep='\t', row.names=T, col.names=T)
################################################################################
############ Table 3
source("table3.R")
out = tbl3.simu(Iter = 5) ## 50 minutes
table3 = round(rbind(do.call("cbind", out[1:4]), do.call("cbind", out[5:8]), 
                     do.call("cbind", out[9:12]), do.call("cbind", out[13:16])),4)
cat("\n", file='OneTable.txt', append=T)
cat("########################## Table 3 ##########################", "\n", file = "OneTable.txt",append=T)
cat("\n", file='OneTable.txt', append = T)
write.table(round(table3,4), file='OneTable.txt', append=T, sep='\t', row.names=F, col.names=T)
