################################################################################
######### Hub Model with Null Component
######### Table 2
################################################################################
## pf = 0.4; pr = 0.02
GenA = function(n,n0,pi,alpha=1){
  A = matrix(NA,n0,n)
  A_lab = vector("list",n0)
  # set of followers except hub itself
  for(i in 1:n0) A_lab[[i]] = seq(n0+i,n,n0)
  for(i in 1:n0){
    A[i,A_lab[[i]]] = runif(length(A_lab[[i]]),0.2,0.4)*alpha
    A[i,-A_lab[[i]]] = runif(n-length(A_lab[[i]]),0,0.2)*alpha
  }
  diag(A) = 1
  return(rbind(pi*alpha,A))
}
##
GenG = function(A,T,rho){
  n0=dim(A)[1]; n=dim(A)[2]
  G = matrix(0,T,n)
  hub = rep(NA,T)
  for(t in 1:T){
    hub[t] = sample(0:(n0-1),1,prob=rho)
    G[t,]=rbinom(n,1,prob=A[hub[t]+1,])
  }
  return(list(G=G,label=hub))
}
##
phub.model = function(i,alpha){
  rr = runif(n0,min=1/(2*n0),max=1*2/n0)
  rho0 = c(0.2, 0.8 * rr/sum(rr))
  A0 = GenA(n,n0,pi = rep(0.05,n), alpha=alpha)
  tempG = GenG(A0,T,rho0)
  G0 = tempG$G
  hublabel = tempG$label
  # estimate A under true label
  Astar = matrix(NA,n0+1,n)
  for(i in 0:n0){
    temp = G0[which(hublabel==i),]
    Astar[(i+1),] = colSums(temp)/nrow(temp)
  }
  # initial A
  A = matrix(NA,n0,n)
  for(i in 1:n0) A[i,] = colSums(G0[,i]*G0)/sum(G0[,i])
  diag(A) = 1; A = rbind(runif(n,0,0.1),A)
  # initial rho
  rho = runif(length(rho0+1)); rho = rho/sum(rho)
  #
  model = phub(G=G0,A=A,rho=rho,lam=0,pen.type="plog")
  #model$mapeA = mean(abs(model$A-A0)/A0)
  mislab = sum(apply(model$post,1,which.max) != (hublabel +1))/T
  rmseAhat = sqrt(sum((model$A-A0)^2)/length(A0))/alpha
  rmseAstar = sqrt(sum((Astar-A0)^2)/length(A0))/alpha
  # rmseRho = sqrt(sum((model$rho-rho0)^2)/length(rho0))/alpha
  # ratioA = rmseAhat/rmseAstar
  cbind(mislab,rmseAhat,rmseAstar)                
}
##
f.set = function(scenario){
  if (scenario == 1) {n0 = 10; n = 100; T = 500}
  if (scenario == 2) {n0 = 10; n = 100; T = 1000}
  if (scenario == 3) {n0 = 10; n = 100; T = 1500}
  if (scenario == 4) {n0 = 10; n = 100; T = 2000}
  if (scenario == 5) {n0 = 10; n = 500; T = 500}
  if (scenario == 6) {n0 = 10; n = 500; T = 1000}
  if (scenario == 7) {n0 = 10; n = 500; T = 1500}
  if (scenario == 8) {n0 = 10; n = 500; T = 2000}
  if (scenario == 9) {n0 = 20; n = 100; T = 500}
  if (scenario == 10) {n0 = 20; n = 100; T = 1000}
  if (scenario == 11) {n0 = 20; n = 100; T = 1500}
  if (scenario == 12) {n0 = 20; n = 100; T = 2000}
  if (scenario == 13) {n0 = 20; n = 500; T = 500}
  if (scenario == 14) {n0 = 20; n = 500; T = 1000}
  if (scenario == 15) {n0 = 20; n = 500; T = 1500}
  if (scenario == 16) {n0 = 20; n = 500; T = 2000}
  return(c(n0 = n0, n = n, T = T))
}
####
tbl2.simu = function(Iter = 5){
  ## parallel computing
  cl = parallel::makeCluster(6)
  doParallel::registerDoParallel(cl)
  out = foreach::foreach(scenario = 1:16, .combine = "rbind",
                         .export = c("GenA","GenG","phub","phub.model","f.set"), .errorhandling = "pass") %dopar% {
                           temp = f.set(scenario)
                           n0 = as.numeric(temp[1])
                           n = as.numeric(temp[2])
                           T = as.numeric(temp[3])
                           colMeans(do.call("rbind",lapply(1:Iter,phub.model,alpha=1)))
                         }
  parallel::stopCluster(cl)
  return(out)
}

